@@ -1,4 +1,4 @@
-# copyright 2011 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
+# copyright 2011-2012 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
 # contact http://www.logilab.fr -- mailto:contact@logilab.fr
 #
 # This program is free software: you can redistribute it and/or modify it under
@@ -26,7 +26,7 @@ from cubicweb.hooks import notification
 
 from cubes.vcsfile import hooks as vcsfile
 
-IGNORE_FILES = set( ('README', 'series', 'status', '.hgignore', '.hgtags') )
+IGNORE_FILES = set( ('README', 'series', 'status', '.hgignore', '.hgtags', 'qsubmitdata') )
 
 def fire(patch, trname, comment):
     patch.cw_adapt_to('IWorkflowable').fire_transition(trname, comment)
@@ -73,8 +73,17 @@ class LinkOrCreatePatchOp(hook.DataOperationMixIn, hook.Operation):
             # patch and this is not a file deletion
             if patch is None or (not patch.is_active() and entity.__regid__ == 'VersionContent'):
                 revision = entity.rev
+                # search originator in the patch content ('# User Chapeau
+                # <Pointu@clowns.org> or use the revision's author
+                originator = revision.author
+                for line in entity.data:
+                    # search for '# User Florent Cayre <florent.cayre@logilab.fr>'
+                    if line.startswith('# User '):
+                        originator = line.split('User ', 1)[1].strip()
+                        originator = originator.decode(entity.repository.encoding)
+                        break
                 patch = self.session.create_entity(
-                    'Patch', originator=revision.author, branch=revision.branch,
+                    'Patch', originator=originator, branch=revision.branch,
                     patch_repository=revision.from_repository, patch_name=entity.file.path)
             patch.set_relations(patch_revision=entity)
             # patch file is being removed: mark it on the patch entity if still
@@ -102,23 +111,30 @@ class LinkOrCreatePatchHook(hook.Hook):
 
 
 class InitPatchCreatorHook(hook.Hook):
+    """When a patch is created, try to set created_by / owned_by relation to the
+    cwuser matching the author.
+
+    This is done by searching author's email, then searching for a user
+    associated to this email.
+    """
     __regid__ = 'vcreview.patch.created'
     __select__ = hook.Hook.__select__ & is_instance('Patch')
-    events = ('before_add_entity',)
+    events = ('after_add_entity',)
     category = 'metadata'
 
     def __call__(self):
-        email = self.entity.originator.rsplit('<')[-1].rsplit('>')[0]
-        rset = self._cw.execute('Any X WHERE X is CWUser, X use_email E, '
-                                'E address %(email)s', {'email': email})
-        relations = []
-        for ueid, in rset:
-            relations.append( (self.entity.eid, ueid) )
-            # ensure user is in the nosy-list
-            self._cw.execute('SET P nosy_list U WHERE P eid %(p)s, U eid %(u)s,'
-                             'NOT P nosy_list U',
-                             {'p': self.entity.eid, 'u': ueid})
-        if relations:
+        author = self.entity.originator
+        author_email = author.rsplit('<', 1)[-1].rsplit('>', 1)[0]
+        rset = self._cw.execute('DISTINCT Any X WHERE X is CWUser, X use_email E, '
+                                'E address ILIKE %(email)s', {'email': author_email})
+        if rset:
+            relations = []
+            for ueid, in rset:
+                relations.append( (self.entity.eid, ueid) )
+                # ensure user is in the nosy-list
+                self._cw.execute(
+                    'SET P nosy_list U WHERE P eid %(p)s, U eid %(u)s,'
+                    'NOT P nosy_list U', {'p': self.entity.eid, 'u': ueid})
             self._cw.add_relations([('created_by', relations),
                                     ('owned_by', relations)])
 
@@ -157,23 +173,29 @@ class SearchPatchStateInstructionOp(hook.DataOperationMixIn,
                     if pstate == 'in-progress':
                         readycandidates[vc.file.path.lower()] = vc.patch
                     elif pstate in ('pending-review', 'reviewed'):
-                        if vc in rev.reverse_from_revision:
+                        if any(vc.eid == x.eid for x in rev.reverse_from_revision):
                             # patch modified in this revision, should go
                             # in-progress unless asking for review again
+                            # or setting as rebased
                             invalidated[vc.file.path.lower()] = vc.patch
                         else:
                             pendingcandidates[vc.file.path.lower()] = vc.patch
                 elif (pstate in vc.patch.non_final_states and
                       vc.__regid__ == 'DeletedVersionContent'):
                     rejectcandidates[vc.file.path.lower()] = vc.patch
-            interestingwords = set( ('ready', 'applied', 'reject', 'fold') )
+            interestingwords = set( ('ready', 'pending',
+                                     'refuse', 'refused',
+                                     'reject', 'rejected',
+                                     'fold', 'folded',
+                                     'apply', 'applied',
+                                     'rebase', 'rebased') )
             interestingwords.update(readycandidates)
             interestingwords.update(pendingcandidates)
             interestingwords.update(rejectcandidates)
             interestingwords.update(invalidated)
             # search for instruction
             for line in rev.description.splitlines():
-                words = [w.lower() for w in re.findall('[-\.\w]+', line.strip())
+                words = [w.lower() for w in re.findall('[\S]+', line.strip())
                          if w.lower() in interestingwords]
                 if len(words) < 2:
                     continue
@@ -210,6 +232,10 @@ class SearchPatchStateInstructionOp(hook.DataOperationMixIn,
                     if patch.cw_adapt_to('IWorkflowable').state == 'reviewed':
                         fire(patch, 'ask review', u'review asked by %s in %s' % (
                                      rev.author, rev.dc_title()))
+                elif words[1] in ('rebase', 'rebased') and words[0] in invalidated:
+                    # patch rebased: nothing has changed from previous revision
+                    # XXX check patch content?
+                    patch = invalidated.pop(words[0])
             for patch in invalidated.itervalues():
                 fire(patch, 'ask rework', u'modified by %s in %s' % (
                     rev.author, rev.dc_title()))
