@@ -17,8 +17,11 @@
 
 import os.path as osp
 from shutil import rmtree
+import tempfile
 
 from logilab.common.shellutils import unzip
+from logilab.common.decorators import classproperty
+
 
 from cubicweb.devtools import testlib
 
@@ -37,20 +40,29 @@ class AutomaticWebTest(testlib.AutomaticWebTest):
 
     @classmethod
     def setUpClass(cls):
-        unzip(cls.repo_path + '.zip', cls.datadir)
+        cls.__tmpdir = tempfile.mkdtemp(prefix='vcreview-test-')
+        unzip(cls.repo_path + '.zip', cls.__tmpdir)
 
     @classmethod
     def tearDownClass(cls):
+        cls.cleanup()
+
+    @classmethod
+    def cleanup(cls):
         try:
-            rmtree(cls.repo_path)
+            rmtree(cls.__tmpdir)
         except:
             pass
 
+    @classproperty
+    def full_repo_path(cls):
+        return osp.join(cls.__tmpdir, cls.repo_path)
+
     # XXX presetup
     def custom_populate(self, how_many, cursor):
         req = self.request()
         req.create_entity('Repository', type=u'mercurial', encoding=u'utf8',
-                          path=self.repo_path)
+                          path=self.full_repo_path)
         assert req.execute('SET X has_patch_repository X')
         self.commit()
         init_vcsrepo(self.repo)
