@@ -26,6 +26,14 @@ class FakeRequest(object):
     def add_css(self, file): pass
 
 class DiffParserTC(TestCase):
+    @classmethod
+    def setUpClass(cls):
+        # in case a previous test set this up
+        try:
+            ENGINE.remove_transform(Diff2HTMLTransform.name)
+        except KeyError:
+            pass
+
     def transform(self, file):
         data = open(self.datapath(file)).read()
         trdata = TransformData(data, 'text/x-diff', 'utf-8')
