@@ -1,4 +1,4 @@
-# copyright 2011 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
+# copyright 2011-2012 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
 # contact http://www.logilab.fr -- mailto:contact@logilab.fr
 #
 # This program is free software: you can redistribute it and/or modify it under
@@ -14,8 +14,11 @@
 # You should have received a copy of the GNU Lesser General Public License along
 # with this program. If not, see <http://www.gnu.org/licenses/>.
 
+from __future__ import with_statement
+
 from os import path as osp
 from shutil import rmtree
+import re
 
 from logilab.common.shellutils import unzip
 from logilab.common.testlib import mock_object
@@ -35,10 +38,15 @@ class PatchCreationHooksTC(CubicWebTC):
 
     @classmethod
     def setUpClass(cls):
+        cls.cleanup()
         unzip(cls.repo_path + '.zip', cls.datadir)
 
     @classmethod
     def tearDownClass(cls):
+        cls.cleanup()
+
+    @classmethod
+    def cleanup(cls):
         try:
             rmtree(cls.repo_path)
         except:
@@ -52,9 +60,23 @@ class PatchCreationHooksTC(CubicWebTC):
         assert req.execute('INSERT EmailAddress E: E address "admin@cwo", U primary_email E '
                     'WHERE U login "admin"')
         self.vcsrepo = self._create_repo(req)
+        self.reviewer1 = self.create_user(req, 'bob', ('users', 'reviewers'),
+                                          email=u'bob@cwo')
         self.commit()
-        init_vcsrepo(self.repo)
-        self.commit()
+        from cubicweb.server import hook
+        from cubicweb.predicates import on_fire_transition
+        class ForceReviewerHook(hook.Hook):
+            __regid__ = 'vcreview.test.forcereviewer'
+            __select__ = hook.Hook.__select__ & on_fire_transition('Patch', 'ask review')
+            events = ('after_add_entity',)
+            def __call__(self):
+                patch = self.entity.for_entity
+                self._cw.execute('SET P patch_reviewer U WHERE P eid %(p)s, U login "admin"',
+                                 {'p': patch.eid, 'u': self._cw.user.eid})
+
+        with self.temporary_appobjects(ForceReviewerHook):
+            init_vcsrepo(self.repo)
+            self.commit()
 
     def _create_repo(self, req):
         vcsrepo = req.create_entity(
@@ -66,9 +88,19 @@ class PatchCreationHooksTC(CubicWebTC):
                               repository_reviewer=req.user)
         return vcsrepo
 
+    def assertMatches(self, got, expected):
+        """ expected is a regular expression """
+        self.assertTrue(re.match(expected, got),
+                        '%r\n does not match:\n %r' % (got, expected))
+
+    def assertEmailEqual(self, email, subject, recipient, content):
+        self.assertEqual(email.subject, subject)
+        self.assertEqual(email.recipients, [recipient])
+        self.assertMatches(email.content, content)
+
     def test_imported_patches(self):
         patches = self.execute('Patch X ORDERBY X')
-
+        # test base patch creation #############################################
         byfile = {}
         bypatch = {}
         for patch in patches.entities():
@@ -78,10 +110,10 @@ class PatchCreationHooksTC(CubicWebTC):
                 byfile.setdefault(iter(files).next(), []).append(patch)
             else:
                 bypatch[patch] = files
-        self.assertEqual(len(byfile), 5, byfile)
+        self.assertEqual(len(byfile), 6, byfile)
         # bypatch check we're correctly following renaming
         self.assertEqual(len(bypatch), 1)
-        self.assertEqual(len(patches), 8)
+        self.assertEqual(len(patches), 9)
 
         folded = bypatch.keys()[0]
         self.assertEqual(folded.cw_adapt_to('IWorkflowable').state, 'folded')
@@ -117,61 +149,80 @@ class PatchCreationHooksTC(CubicWebTC):
         self.assertEqual(history[-1].previous_state.name, 'pending-review')
         self.assertEqual(history[-1].new_state.name, 'in-progress')
 
+        # test nosy-list / notifications #######################################
         self.assertTrue(patch3_2.nosy_list)
         self.assertTrue(patch4.nosy_list)
-        self.assertEqual(len(MAILBOX), 6, MAILBOX)
-        self.assertEqual(MAILBOX[0].subject, '[patch repository] patch pending-review: patch3.diff')
-        self.assertEqual(MAILBOX[0].recipients, ['admin@cwo'])
-        self.assertMultiLineEqual(MAILBOX[0].content, u'''
+        self.assertEqual(len(MAILBOX), 8, (len(MAILBOX), MAILBOX))
+
+        self.assertEmailEqual(MAILBOX[0],
+                              '[patch repository] patch pending-review: patch3.diff',
+                              'admin@cwo',
+                              u'''
 cubicweb changed status from <in-progress> to <pending-review> for entity
 'patch3.diff'
 
-review asked by Sylvain Th\xe9nault <sylvain.thenault@logilab.fr> in
+review asked by Sylvain Th.*nault <sylvain.thenault@logilab.fr> in
 #10:9cf50f15d9a7
 
 url: http://testing.fr/cubicweb/patch/%s\n''' % patch3_2.eid)
 
-        self.assertEqual(MAILBOX[1].subject, '[patch repository] patch assigned: patch3.diff')
-        self.assertEqual(MAILBOX[1].recipients, ['admin@cwo'])
-        self.assertMultiLineEqual(MAILBOX[1].content, u'''
+        self.assertEmailEqual(MAILBOX[1],
+                              '[patch repository] patch assigned: patch3.diff',
+                              'admin@cwo',
+                              u'''
 You have been assigned for review of patch patch3.diff
 
 url: http://testing.fr/cubicweb/patch/%s\n''' % patch3_2.eid)
 
-        self.assertEqual(MAILBOX[2].subject, '[patch repository] patch rejected: patch3.diff')
-        self.assertEqual(MAILBOX[2].recipients, ['admin@cwo'])
-        self.assertMultiLineEqual(MAILBOX[2].content, u'''
+        self.assertEmailEqual(MAILBOX[2],
+                              '[patch repository] patch rejected: patch3.diff',
+                              'admin@cwo',
+                              u'''
 cubicweb changed status from <pending-review> to <rejected> for entity
 'patch3.diff'
 
-rejected by Sylvain Th\xe9nault <sylvain.thenault@logilab.fr> in #15:a5c0568f81db
+rejected by Sylvain Th.*nault <sylvain.thenault@logilab.fr> in #15:0fc9a74c0ce9
 
 url: http://testing.fr/cubicweb/patch/%s\n''' % patch3_2.eid)
 
-        self.assertEqual(MAILBOX[3].subject, '[patch repository] patch pending-review: patch4.diff')
-        self.assertEqual(MAILBOX[3].recipients, ['admin@cwo'])
-        self.assertMultiLineEqual(MAILBOX[3].content, u'''
+        self.assertEmailEqual(MAILBOX[3],
+                              '[patch repository] patch pending-review: patch4.diff',
+                              'admin@cwo',
+                              u'''
 cubicweb changed status from <in-progress> to <pending-review> for entity
 'patch4.diff'
 
-review asked by Julien Cristau <julien.cristau@logilab.fr> in #17:2f2bad1f3f12
+review asked by Bob Cornelieus <bob@cwo> in #17:f45b1e15a66e
 
 url: http://testing.fr/cubicweb/patch/%s\n''' % patch4.eid)
 
-        self.assertEqual(MAILBOX[4].subject, '[patch repository] patch assigned: patch4.diff')
-        self.assertEqual(MAILBOX[4].recipients, ['admin@cwo'])
-        self.assertMultiLineEqual(MAILBOX[4].content, u'''
+        self.assertEmailEqual(MAILBOX[4],
+                              '[patch repository] patch pending-review: patch4.diff',
+                              'bob@cwo',
+                              u'''
+cubicweb changed status from <in-progress> to <pending-review> for entity
+'patch4.diff'
+
+review asked by Bob Cornelieus <bob@cwo> in #17:f45b1e15a66e
+
+url: http://testing.fr/cubicweb/patch/%s\n''' % patch4.eid)
+
+        self.assertEmailEqual(MAILBOX[5],
+                         '[patch repository] patch assigned: patch4.diff',
+                              'admin@cwo',
+                              u'''
 You have been assigned for review of patch patch4.diff
 
 url: http://testing.fr/cubicweb/patch/%s\n''' % patch4.eid)
 
-        self.assertEqual(MAILBOX[5].subject, '[patch repository] patch in-progress: patch4.diff')
-        self.assertEqual(MAILBOX[5].recipients, ['admin@cwo'])
-        self.assertMultiLineEqual(MAILBOX[5].content, u'''
+        self.assertEmailEqual(MAILBOX[6],
+                              '[patch repository] patch in-progress: patch4.diff',
+                              'admin@cwo',
+                              u'''
 cubicweb changed status from <pending-review> to <in-progress> for entity
 'patch4.diff'
 
-modified by Julien Cristau <julien.cristau@logilab.fr> in #19:650cce0df5e2
+modified by Julien Cristau <julien.cristau@logilab.fr> in #19:0a504eb04e48
 
 url: http://testing.fr/cubicweb/patch/%s\n''' % patch4.eid)
 
@@ -181,9 +232,10 @@ url: http://testing.fr/cubicweb/patch/%s\n''' % patch4.eid)
         done = req.create_entity('Task', title=u'done', reverse_has_activity=patch3_2)
         self.commit()
         self.assertEqual(len(MAILBOX), 2)
-        self.assertEqual(MAILBOX[1].subject, '[patch repository] patch task: patch3.diff - done')
-        self.assertEqual(MAILBOX[1].recipients, ['admin@cwo'])
-        self.assertMultiLineEqual(MAILBOX[1].content, u'''
+        self.assertEmailEqual(MAILBOX[1],
+                              '[patch repository] patch task: patch3.diff - done',
+                              'admin@cwo',
+                              u'''
 done
 
 
@@ -223,11 +275,24 @@ url: http://testing.fr/cubicweb/task/%s\n''' % done.eid)
         #self.assertEqual(patch3_2.applied_at[0].eid, patch3_2.revisions[-1].rev.eid) XXX
 
         self.assertEqual(len(MAILBOX), 2, MAILBOX)
-        self.assertEqual(MAILBOX[0].subject, '[patch repository] patch reviewed: patch3.diff')
-        self.assertEqual(MAILBOX[1].subject, '[patch repository] patch applied: patch3.diff')
-        self.assertEqual(MAILBOX[0].recipients, ['admin@cwo'])
-        self.assertEqual(MAILBOX[1].recipients, ['admin@cwo'])
-        self.assertMultiLineEqual(MAILBOX[1].content, u'''
+        self.assertEmailEqual(MAILBOX[0],
+                              '[patch repository] patch reviewed: patch3.diff',
+                              'admin@cwo',
+                              '''
+admin changed status from <pending-review> to <reviewed> for entity
+'patch3.diff'
+
+
+
+remaining tasks:
+
+- todo
+
+url: http://testing.fr/cubicweb/patch/%s\n''' % patch3_2.eid)
+        self.assertEmailEqual(MAILBOX[1],
+                              '[patch repository] patch applied: patch3.diff',
+                              'admin@cwo',
+                              u'''
 admin changed status from <reviewed> to <applied> for entity
 'patch3.diff'
 
@@ -240,7 +305,9 @@ remaining tasks:
 url: http://testing.fr/cubicweb/patch/%s\n''' % patch3_2.eid)
 
         self.vcsrepo._cw = self.request()
-        # test mime type enforcement
+
+        # other meta-data ######################################################
+        # mime type enforcements for special files
         self.assertEqual(
             self.vcsrepo.versioned_file('', 'patch_no_ext').head.data_format,
             'text/x-diff')
@@ -250,6 +317,21 @@ url: http://testing.fr/cubicweb/patch/%s\n''' % patch3_2.eid)
         self.assertEqual(
             self.vcsrepo.versioned_file('', 'README').head.data_format,
             'text/plain')
+        # author from patch content
+        self.assertEqual(folded.originator, 'The Patch Author <admin@cwo>')
+        self.assertEqual(folded.creator.login, 'admin')
+        # author from email
+        patch = byfile['patch4.diff'][0]
+        self.assertEqual(patch.originator, 'Bob Cornelieus <bob@cwo>')
+        self.assertEqual(patch.creator.login, 'bob')
+        # unmatched user
+        patch = byfile['patch1.diff'][0]
+        self.assertMatches(patch.originator, u'Sylvain Th.*nault <sylvain.thenault@logilab.fr>')
+        self.assertEqual(patch.creator, None)
+
+        # check email match is case-insensitive
+        patch = byfile['patch5.diff'][0]
+        self.assertEqual(patch.creator.login, 'bob')
 
 PatchCreationHooksTC.repo_path = osp.join(PatchCreationHooksTC.datadir, u'patchrepo')
 
