@@ -31,11 +31,11 @@ _affk = uicfg.autoform_field_kwargs
 def patch_applied_at_choices(form, field):
     entity = form.edited_entity
     # first see if its specified by __linkto form parameters
-    linkedto = formfields.relvoc_linkedto(entity, 'applied_at', 'subject')
+    linkedto = field.relvoc_linkedto(form)
     if linkedto:
         return linkedto
     # it isn't, check if the entity provides a method to get correct values
-    vocab = formfields.relvoc_init(entity, 'applied_at', 'subject')
+    vocab = field.relvoc_init(form)
     assert entity.has_eid()
     for rev in form._cw.execute(
         'Any REV,REVD ORDERBY REV DESC LIMIT 20 WHERE X patch_repository PR, '
